#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"

#define VENDOR_RQ_COUNT_STAT    0
#define VENDOR_RQ_GET_LIGHT 	1
#define VENDOR_RQ_SET_LIGHT 	2
#define VENDOR_RQ_GET_SET_LIGHT 3
#define VENDOR_RQ_RESET_COUNT	4
#define VENDOR_RQ_TOGGLE_LED	5
#define VENDOR_RQ_STAT_BRIGHT	6
#define VENDOR_RQ_RED		7	

uint16_t read_adc(uint8_t channel)
{
    ADMUX = 0b01000000; // เลือกใช้แรงดันอ้างอิงจากขา AREF
    ADMUX |= channel;   // ตั้ง MUX ให้ชี้ไปยัง channel
    ADCSRA = 0b11000110;
    //         ||   -+-
    //         ||    |
    //         ||    +-> ปรับคล็อกของ ADC ให้เป็น 1/64 ของคล็อกระบบ
    //         |+->เริ่มแปลงค่าแอนะล็อกเป็นดิจิทัล
    //         |
    //         +-> Enable ADC

    while ((ADCSRA & (1<<ADSC))) // รอจนบิต ADSC (บิต 6) กลายเป็น 0
        ;
    return ADCL + ADCH*256;  // ผลลัพธ์ถูกเก็บอยู่ในรีจีสเตอร์ ADCL และ ADCH
}

uint16_t count,light,countdown=0,delay=0;
usbMsgLen_t usbFunctionSetup(uchar data[8])
{
	uint16_t temp;
	usbRequest_t *rq = (void *)data;
  	static uchar dataBuffer[2];  /* must stay valid when usbFunctionSetup returns */

	if (rq->bRequest == VENDOR_RQ_COUNT_STAT)
	{
		dataBuffer[0] = count;
		if(PINC & 0b00000001)
			dataBuffer[1] = 1;
		else
			dataBuffer[1] = 0;
		usbMsgPtr = dataBuffer;
    	   	return 2;
    	}
	if (rq->bRequest == VENDOR_RQ_GET_LIGHT)
	{
		temp = read_adc(4);
		dataBuffer[0] = temp;
		dataBuffer[1] = temp>>8;
		usbMsgPtr = dataBuffer;
		return 2;
	} 
	if(rq->bRequest == VENDOR_RQ_SET_LIGHT)
	{
		light = rq->wValue.bytes[0]+256*rq->wValue.bytes[1];
		return 0;
	}
	if (rq->bRequest == VENDOR_RQ_GET_SET_LIGHT)
	{
		dataBuffer[0] = light;
		dataBuffer[1] = light>>8;
		usbMsgPtr = dataBuffer;
    	   	return 2;
    	}
	if(rq->bRequest == VENDOR_RQ_RESET_COUNT)
	{
		count = 0;
		return 0;
	}
	if(rq->bRequest == VENDOR_RQ_TOGGLE_LED)
	{
		if(PINC & 0b00100000)
		{
			PORTC &= 0b11011111;
			light = 0;
		}
		else
		{
			PORTC |= 0b00100000;
			light = 800;
		}
		return 0;
	}
	if(rq->bRequest == VENDOR_RQ_STAT_BRIGHT)
	{
		dataBuffer[0] = 1;
		if(PINC & 0b00100000)
			dataBuffer[0] = 1;
		else
			dataBuffer[0] = 0;
		usbMsgPtr = dataBuffer;
		return 1;
	}
	if(rq->bRequest == VENDOR_RQ_RED)
	{
		if(!(PINC & 0b00100000))
			if(PINC & 0b00000001)
				PORTC &= 0b11111110;
			else
				PORTC |= 0b00000001;
		return 0;
	}
   
    return 0;   /* default for not implemented requests: return no data back to host */
}

main()
{
	DDRC &= 0b11100111;  // กำหนดให้ขา PC4PC3 ทำหน้าที่อินพุท(วงจรวัดแสงและสวิตรช์)
	DDRC |= 0b00100111;  // กำหนดให้ขา PC 0-2 เป็นเอาท์พุท(หลอดไฟสามดวง)
	PORTC |=0b00001000;
	count = 0;	
	light = 0;
	usbInit();
	usbDeviceDisconnect();  /* enforce re-enumeration, do this while interrupts are disabled! */
	_delay_ms(300);
   	usbDeviceConnect();
    	sei();
    	for(;;)
	{                /* main event loop */
       		usbPoll();
		if ((PINC & 0b00001000)==0)
		{
			_delay_ms(10);
			while((PINC & 0b00001000)==0);
			_delay_ms(10);
			if(light != 0)
				light -= 150;
			if(light < 350)
				light = 800;
			PORTC &= 0b11111001;
			//PORTC &= 0b11011111;
		}
		switch (light)
		{
			case 800 : PORTC |= 0b00000110; break;
			case 650 : PORTC |= 0b00000100;	break;
			case 500 : PORTC |= 0b00000010; break;
			case 350 : PORTC |= 0b00000000; break;
		}
		if(read_adc(4)<light)
        	{
			if(delay == 15)
			{
           			countdown = 500;
				PORTC |= 0b00000001;
				count++;
				delay = 0;
			}
			delay++;
        	}
        	countdown--;
		if(countdown == 0)
		{
			PORTC &= 0b11111110;
		}
		_delay_ms(10);		
    	}
    	return 0;
}
